# -*- encoding: utf-8 -*-

from django.db import models
from models import PermissionMeta, Pays, Region
from implantation import TypeImplantation, Implantation
from django_historique.models import History
from django.db.models.signals import pre_save, pre_delete
from django.conf import settings


EE_STATUT_CHOICES = (('N', 'Non membre'), \
                     ('A', 'Associé'), \
                     ('T', 'Titulaire'), \
                     ('C', 'Candidat'))

E_QUALITE_CHOICES = (('ESR', 'Etablissement d\'enseignement superieur et de recherche'), \
                     ('CIR', 'Centre ou institution de recherche'), \
                     ('RES', 'Reseaux'))

R_GENRE_CHOICES = (('H', 'Homme'), ('F', 'Femme'))


## Etablissement
class EtablissementGeneral (models.Model):
    __metaclass__ = PermissionMeta

    nom = models.CharField(max_length=255, db_column = 'intit_lg_fr',
                           verbose_name='Intitulé (français)')
    nomLocal = models.CharField(max_length=255, db_column = 'intit_lg_origine',
                                null = True, blank = True,
                                verbose_name = 'Intitulé (langue d\'origine)')
    adresse = models.CharField(max_length=255, null = True)
    codePostal = models.CharField(max_length=32, db_column = 'code_postal',
                                  null = True, verbose_name = 'Code postal')
    cedex = models.CharField(max_length=32, null = True, blank = True)
    province = models.CharField(max_length=64, null = True, blank = True)
    ville = models.CharField(max_length=128, db_column = 'ville_lg_fr',
                             null = True, verbose_name = 'Ville (français)')
    villeLocal = models.CharField(max_length = 128,
                                  db_column = 'ville_lg_origine', null = True,
                                  verbose_name = 'Ville (langue d\'origine)',
                                  blank = True)
    pays = models.ForeignKey(Pays, db_column = 'pays_iso2')
    region = models.ForeignKey(Region, db_column = 'id_region',
                               null = True,
                               verbose_name = 'Region de rattachement')
    tel1 = models.CharField(max_length=32, null = True,
                            verbose_name = 'Telephone 1')
    poste1 = models.CharField(max_length=16, null = True,
                              verbose_name = 'Poste', blank = True)
    tel2 = models.CharField(max_length=32, null = True,
                            verbose_name = 'Telephone 2', blank = True)
    poste2 = models.CharField(max_length=16, null = True,
                              verbose_name = 'Poste', blank = True)
    fax = models.CharField(max_length=32, null = True, blank = True)
    fax2 = models.CharField(max_length=32, null = True, blank = True,
                            verbose_name = "Fax 2")
    courriel = models.EmailField(max_length=128, null = True, blank = True)
    url = models.CharField(max_length=128, db_column = 'site_internet',
                           null = True, verbose_name = 'Site internet',
                           blank = True)
    historique = models.TextField(null = True, verbose_name = 'Historique RG',
                                  blank = True)
    etudiants = models.IntegerField(db_column = 'nb_etud', null = True,
                                    verbose_name = 'Nombre d\'étudiants',
                                    blank = True)
    anRefEtudiants = models.IntegerField(db_column = 'nb_etud_annee_obs',
                                         null = True,
                                         verbose_name = 'Année de référence (Étudiants)',
                                         blank = True)
    enseignants = models.IntegerField(db_column = 'nb_ens', null = True,
                                      verbose_name = 'Nombre d\'enseignants',
                                      blank = True)
    anRefEnseignants = models.IntegerField(db_column = 'nb_ens_annee_obs',
                                           null = True,
                                           verbose_name = 'Année de référence (Enseignants)',
                                           blank = True)
    chercheurs = models.IntegerField(db_column = 'nb_cherch', null = True,
                                     verbose_name = 'Nombre de chercheurs',
                                     blank = True)
    anRefChercheurs = models.IntegerField(db_column = 'nb_cherch_annee_obs',
                                          null = True,
                                          verbose_name = 'Année de référence (Chercheurs)',
                                          blank = True)
    miseAJour = models.DateField(db_column = 'datemaj', auto_now = True,
                                 verbose_name = "Date de mise à jour")
    actif = models.NullBooleanField()
    codaName = models.CharField(max_length=36, db_column = 'coda_name',
                                verbose_name = 'Nom long (Coda)')
    codaSname = models.CharField(max_length=20, db_column = 'coda_sname',
                                 verbose_name = 'Nom court (Coda)')
    historiqueAnnuaire = models.TextField(db_column = 'rub', null = True,
                                          verbose_name = 'Historique annuaire', blank = True)
    descriptif = models.TextField(null = True, blank = True)
    commentaire = models.TextField(null = True, verbose_name = 'Commentaires',
                                   blank = True)
    qualite = models.CharField(max_length = 3, null = True,
                               verbose_name = u'Qualité de l\'établissement',
                               choices = E_QUALITE_CHOICES, blank = True)
    prive = models.NullBooleanField(verbose_name = 'Privé')
    adhesion = models.IntegerField(null = True,
                                   verbose_name = 'Année d\'adhesion',
                                   blank = True)
    implantation = models.ForeignKey(Implantation,
                                     db_column = 'implantation', null = True,
                                     verbose_name = 'Implantation de rattachement',
                                     blank = True)
    sigle = models.CharField(max_length=16, null = True, blank = True)
    statut = models.CharField(max_length = 1, null = True,
                              choices = EE_STATUT_CHOICES, blank = True)
    statutEffectif = models.DateField(db_column = 'date_effectif', null = True,
                                      verbose_name = "Effectif le")
    commentaireStatut = models.TextField(null = True,
                                         verbose_name = 'Commentaires statut',
                                         blank = True)

    class Meta:
        abstract = True

class Etablissement (EtablissementGeneral):
    id = models.AutoField(primary_key=True, db_column = 'id_etablissement')
    def __unicode__ (self):
        return self.nom
    class Meta:
        db_table = u'etablissement'
        ordering = ["nom"]

class EtablissementHistory(EtablissementGeneral, History):
    class Meta:
        db_table = u'histo_etablissement'


## Etat Etablissement
# Supprimé, inutile
#class EtatEtablissementGeneral (models.Model):
#    __metaclass__ = PermissionMeta
#
#    etablissement = models.OneToOneField(Etablissement,
#            db_column = 'id_etablissement', null = True)
#    statut = models.CharField(max_length = 1, null = True,
#            choices = EE_STATUT_CHOICES, blank = True)
#    dateEffectif = models.DateField(db_column = 'date_effectif', null = True,
#            verbose_name = "Effectif le")
#    miseAJour = models.DateField(db_column = 'date_maj', auto_now = True)
#    commentaire = models.TextField(null = True,
#            verbose_name = 'Commentaires statut', blank = True)
#    class Meta:
#        abstract = True
#
#class EtatEtablissement(EtatEtablissementGeneral):
#    id = models.AutoField(primary_key = True, db_column = 'id_etat')
#    def __unicode__ (self):
#        return self.etablissement.nom + ' - ' + str(self.statut)
#    class Meta:
#        db_table = u'etat_etablissement'
#        ordering = ["etablissement__nom"]
#
#class EtatEtablissementHistory(History, EtatEtablissementGeneral):
#    class Meta:
#        db_table = u'histo_etat_etablissement'
#
#def etatetablissementhistory_save (sender, instance, signal, *args, **kwargs):
#    history = EtatEtablissementHistory ()
#    history.save_log(instance)
#pre_save.connect(etatetablissementhistory_save, sender=EtatEtablissement)
#pre_delete.connect(etatetablissementhistory_save, sender=EtatEtablissement)


## Responsable
class ResponsableBase(models.Model):
    __metaclass__ = PermissionMeta

    id = models.AutoField(primary_key=True, db_column = 'id_responsable')
    salutation = models.CharField(max_length=128)
    nom = models.CharField(max_length=128)
    prenom = models.CharField(max_length=128, verbose_name = 'Prénom')
    fonction = models.CharField(max_length=128)
    sousfonction = models.CharField(max_length=128, blank = True, \
            verbose_name = 'Sous-fonction')
    courriel = models.CharField(max_length=128, blank = True)
    mobile = models.CharField(max_length=32, blank = True, \
            verbose_name = 'Numéro de téléphone portable ')
    priorite = models.CharField(max_length=3, verbose_name = 'Priorité')
    genre = models.CharField(max_length=1, choices=R_GENRE_CHOICES)
    commentaire = models.TextField(verbose_name = 'Commentaires', blank = True)
    actif = models.BooleanField(editable = False)
    class Meta:
        abstract = True

class Responsable(ResponsableBase):
    etablissement = models.ForeignKey(Etablissement, db_column='id_etablissement')
    class Meta:
        db_table = u'etablissement_responsable'
